/******************************************/
/* pick.c                                 */
/*                                        */
/* by y.tosa (71320.1430)                 */
/*                                        */
/* May 3, 1991  v1.01                     */
/******************************************/
/* allows to pick various number of rows  */
/*************************************************************************/
/* Note that ANSI.SYS comes with DOS may not allow you to have more than */
/* 25 rows, even though you specify 28, 43, or 50 rows.                  */
/* Disable ANSI.SYS or download smode.arc in IBM sys/video lib.          */
/* It contains NANSI.SYS and NANSI286.SYS.                               */
/*************************************************************************/
/* How to use                             */
/* pick number_of_rows    i.e. pick 50    */
/* or                                     */
/* pick                                   */
/******************************************/
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>

unsigned get_active_page(void);
void check_vga(unsigned *);
void select(unsigned scanline);
void set_mode(unsigned mode);
void load(unsigned page, unsigned character);

union REGS rg;

#define VGA 8
#define EGA 4

#define VIDEO 0x10

/* scan lines */
#define S200 0
#define S350 1
#define S400 2

/* font size */
#define C8   0x12
#define C14  0x11
#define C16  0x14

/* vide mode */
#define M0  0
#define M2  2

void main(int argc, char **argv)
{
	int i;
	unsigned page;
	unsigned DISPLAY;

	printf("\nPICK v1.0 by Y. Tosa (71320,1430)\n");

	check_vga(&DISPLAY); 			 /* make sure that ega/vga color is active */

	page = get_active_page();

	if (argc < 2)
	{
		printf("\nHow many rows per page?");

		if (DISPLAY == VGA)
		{
			printf("\nType the number (0, 21, 25, 28, 43, 50) and hit <Enter> :");
			scanf("%d%*c", &i);
		}
		if (DISPLAY == EGA)
		{
			printf("\nType the number (0, 21, 25, 43) and hit <ENTER> :");
			scanf("%d%*c", &i);
			if (i == 28 || i == 50)
			{
				printf("\nOnly one of 0, 21, 25, 43.  Try again.\n");
				exit(1);
			}
		}
	}
	else
	{
		i = atoi(argv[1]);
		if (DISPLAY == EGA)
		{
			if (i == 28 || i == 50)
			{
				printf("\nOnly one of 0, 21, 25, 43.  Try again.\n");
				exit(1);
			}
		}
	}

	switch(i)
	{
		case 0:
						select(S400);
						set_mode(M0);
						break;

		case 21:
						select(S350);
						set_mode(M2);
						load(page, C16);
						break;

		case 25:
						if (DISPLAY == VGA)
						{
							select(S400);
							set_mode(M2);
							load(page, C16);
						}
						else
						{
							select(S400);
							set_mode(M2);
							load(page, C14);
						}
						break;

		case 28:
						select(S400);
						set_mode(M2);
						load(page, C14);
						break;

		case 43:
						select(S350);
						set_mode(M2);
						load(page, C8);
						break;

		case 50:
						select(S400);
						set_mode(M2);
						load(page, C8);
						break;

		default:
						printf("\nSelect one of 0, 21, 25, 28, 43, 50.\n");
						printf("Try again.");
						break;
	}

}

void check_vga(unsigned* DISPLAY)
{
	rg.h.ah = 0x1a;
	rg.h.al = 0x00;
	int86(VIDEO, &rg, &rg);

	*DISPLAY = rg.h.bl;
	if ((*DISPLAY != 4) && (*DISPLAY != 8))
	{
		printf("\nOnly EGA or VGA color mode supported.  Sorry.");
		exit(1);
	}
	if (*DISPLAY == VGA)
		printf("\nThe active display is VGA color.\n");
	if (*DISPLAY == EGA)
		printf("\nThe active display is EGA color.\n");
}

unsigned get_active_page(void)
{
	rg.h.ah = 0x0f;
	int86(VIDEO, &rg, &rg);
	return(rg.h.bh);
}

void select(unsigned scanline)
{
	rg.h.ah = 0x12;
	rg.h.al = scanline;
	rg.h.bl = 0x30;
	int86(VIDEO, &rg, &rg);
	if (rg.h.al == 0x00)
	{
		printf("\nEGA/VGA not active!");
		exit(1);
	}
}


void set_mode(unsigned mode)
{
	rg.h.ah = 0x00;
	rg.h.al = mode;
	int86(VIDEO, &rg, &rg);
	if (rg.h.al != mode )
	{
		printf("\nError in set_mode!");
		exit(1);
	}
}

void load(unsigned page, unsigned character)
{
	rg.h.ah = 0x11;	       /* load character generator */
	rg.h.al = character;   /* text mode font value */
	rg.h.bl = page;
	int86(VIDEO, &rg, &rg);
}
